/*
 * Licensed Materials - Property of HCL
 * UrbanCode Deploy
 * (c) Copyright HCL Technologies Ltd. 2018. All Rights Reserved.
 */
package com.urbancode.air.plugin.ispw.versions

import groovy.json.JsonBuilder
import org.apache.http.HttpResponse
import org.apache.log4j.Logger

import com.urbancode.air.plugin.ispw.http.CESClient
import com.urbancode.air.plugin.ispw.models.SetInfo

/*
 * Client to handle ISPW component version related operations.
 * Both Assignments and Releases are logical version groupings in ISPW.
 * This client generates, promotes, deploys, and regresses Assignments and Releases.
 * Operations against Assignments and Releases generate Sets that execute in the background.
 */
abstract class VersionClient extends CESClient {

    public VersionClient(Properties props) {
        super(props)    // Configure CES connection
    }

    /* Promote an ISPW Assignment/Release from a given level */
    public Map<String, String> promote(String id, String level) {
        SetInfo setInfo = getSetInfo() // Get SetInfo from the relevant Properties in CESClient
        String uri = String.format(getPromoteUri(), serverId, id) + URLEncoder.encode(level, "UTF-8")
        HttpResponse response = restHelper.doPostRequest(uri, setInfo.getJson())
        Map<String, String> responseMap = restHelper.parseResponse(response)
        return responseMap
    }

    /* Deploy an ISPW Assignment/Release to a given level */
    public Map<String, String> deploy(String id, String level) {
        SetInfo setInfo = getSetInfo() // Get SetInfo from the relevant Properties in CESClient
        String uri = String.format(getDeployUri(), serverId, id) + URLEncoder.encode(level, "UTF-8")
        HttpResponse response = restHelper.doPostRequest(uri, setInfo.getJson())
        Map<String, String> responseMap = restHelper.parseResponse(response)
        return responseMap
    }

    /* Regress an ISPW Assignment/Release from a given level */
    public Map<String, String> regress(String id, String level) {
        SetInfo setInfo = getSetInfo() // Get SetInfo from the relevant Properties in CESClient
        String uri = String.format(getRegressUri(), serverId, id) + URLEncoder.encode(level, "UTF-8")
        HttpResponse response = restHelper.doPostRequest(uri, setInfo.getJson())
        Map<String, String> responseMap = restHelper.parseResponse(response)
        return responseMap
    }

    /* Regress an ISPW Assignment/Release from a given level */
    public Map<String, String> generateTasks(String id, String level) {
        SetInfo setInfo = getSetInfo() // Get SetInfo from the relevant Properties in CESClient
        String uri = String.format(getGenerateUri(), serverId, id) + URLEncoder.encode(level, "UTF-8")
        HttpResponse response = restHelper.doPostRequest(uri, setInfo.getJson())
        Map<String, String> responseMap = restHelper.parseResponse(response)
        return responseMap
    }

    /*
     * Abstract methods that will be overridden to get specific endpoints.
     */
    protected abstract String getPromoteUri()

    protected abstract String getDeployUri()

    protected abstract String getRegressUri()

    protected abstract String getGenerateUri()
}
